from django.core.management.base import BaseCommand

from djx_cmds.services.clean_migration import CleanMigration
from djx_cmds.services.scaffolding_service import ScaffoldingService


class Command(BaseCommand):
    help = ''

    def add_arguments(self, parser):
        parser.add_argument('--name', type=str, help='name')

    def handle(self, *args, **options):
        name = options['name']
        CleanMigration(name).run()



from django.core.management.base import BaseCommand

from djx_cmds.services.clean_migration import CleanMigration
from djx_cmds.services.extract_models_service import ExtractModelsService
from djx_cmds.services.scaffolding_service import ScaffoldingService


class Command(BaseCommand):
    help = ''

    def handle(self, *args, **options):
        content = ExtractModelsService().run()
        self.stdout.write(content)



import os

import inflection
from django.conf import settings
from django.contrib.auth import get_user_model
from django.db.models import ManyToOneRel

from djx_cmds.format_with_jinja import format_with_jinja
from djx_cmds.services.import_object import ImportObject


class ScaffoldingService:

    def __init__(self, name, actions):
        self.name = name
        self.actions = actions

    def run(self):
        model_data = ImportObject().import_model(self.name)
        model_name = model_data['name']
        model_module = model_data['module']

        model = ImportObject().get_model(model_name)
        app_name = ImportObject.get_app(model)
        fields = ImportObject().get_model_fields(model)
        many_to_one_relations = self.get_many_to_one_relations(fields)
        user_fields = self.get_user_model_fields(fields)

        self.create_serializer(model_name, model_module, fields, user_fields, many_to_one_relations, app_name)
        self.create_views(model_name, model_module, fields, user_fields, many_to_one_relations, app_name)

    def create_serializer(self, model_name, model_module, fields, user_fields, many_to_one_relations, app_name):

        serializer_content = format_with_jinja(
            {
                'model_name': model_name,
                "methods": list(self.actions),
                "fields": fields,
                "model_module": model_module,
                "user_fields": user_fields,
                "many_to_one_relations": many_to_one_relations
            },
            'serializer.txt'
        )

        serializer_file_name = inflection.underscore(f"{model_name}_serializers.py")

        serializer_file_path = os.path.join(settings.BASE_DIR, app_name, 'serializers', serializer_file_name)

        self.create_file(file_path=serializer_file_path, file_content=serializer_content)

    @staticmethod
    def create_file(file_path, file_content, mode='w'):
        dirname = os.path.dirname(file_path)
        if not os.path.exists(dirname):
            os.makedirs(dirname)
        with open(file_path, mode, encoding='utf-8') as f:
            f.write(file_content)

    @staticmethod
    def create_urls(app_name, model_name):
        dash = inflection.dasherize(inflection.underscore(model_name))
        register = f"router.register('{dash}', {model_name}ViewSet, basename='{dash}')"
        file_path = os.path.join(settings.BASE_DIR, app_name, 'urls.py')
        with open(file_path, 'r') as f:
            content = f.read()
        tag = "# registration"
        import_str = f"from {app_name}.views.{inflection.underscore(model_name)}_views import {model_name}ViewSet"
        content = content.replace(f"{tag}", f'{register}\n{tag}')
        content = f"{import_str}\n{content}"
        with open(file_path, 'w') as f:
            content = f.write(content)

    @staticmethod
    def get_user_model_fields(fields):
        UserModel = get_user_model()
        return [element["name"] for element in fields if element["cls"].related_model == UserModel]

    @staticmethod
    def get_many_to_one_relations(array):
        many_relations = []
        for item in array:
            if isinstance(item['cls'], ManyToOneRel):
                many_relations.append(item['name'])
        return many_relations

    def create_views(self, model_name, model_module, fields, user_fields, many_to_one_relations, app_name):
        model = ImportObject().get_model(model_name)
        app_name = ImportObject.get_app(model)
        view_content = format_with_jinja(
            {
                "app_name": app_name,
                'model_name': model_name,
                'model_name_snake_case': inflection.underscore(model_name),
                'actions': self.actions,
                "methods": list(self.actions),
                "fields": fields,
                "model_module": model_module,
                "user_fields": user_fields,
                "many_to_one_relations": many_to_one_relations

            },
            'view.txt')

        view_file_name = inflection.underscore(f"{model_name}_views.py")

        view_file_path = os.path.join(settings.BASE_DIR, app_name, 'views', view_file_name)
        self.create_file(file_path=view_file_path, file_content=view_content)
        self.create_urls(app_name, model_name)



from django.core.management.base import BaseCommand
from djx_cmds.services.models_to_angular_service import ModelsToAngular


class Command(BaseCommand):
    help = ''

    def add_arguments(self, parser):
        parser.add_argument(
            '--file_path',
            type=str,
            help='The path to the directory or file',
            required=True
        )

    def handle(self, *args, **options):
        file_path = options['file_path']
        ModelsToAngular(file_path).run()



import os
from pathlib import Path

from django.conf import settings


class InstallAppService:

    def __init__(self, app_name):
        self.app_name = app_name

    def run(self):
        self.install_app()
        self.install_url()

    def install_app(self):
        install_app_tag = "# installAppRegistration"
        replace_value = f'"{self.app_name}",\n\t{install_app_tag}'
        settings_module = os.environ.get('DJANGO_SETTINGS_MODULE')
        settings_path = Path(settings_module.replace('.', '/') + '.py')
        self.find_and_replace(settings_path, install_app_tag, replace_value)

    def install_url(self):
        install_url_tag = "# AppUrlRegistration"
        urls_module = settings.ROOT_URLCONF
        url_path = Path(urls_module.replace('.', '/') + '.py')

        replace_value = f"path('api/', include('{self.app_name}.urls')),\n\t{install_url_tag}"
        self.find_and_replace(url_path, install_url_tag, replace_value)

    @staticmethod
    def find_and_replace(file_path, tag, replace_value):
        with open(file_path, 'r') as file:
            content = file.read()
        updated_content = content.replace(tag, replace_value)
        with open(file_path, 'w') as file:
            file.write(updated_content)



import django


class ImportObject:

    @staticmethod
    def format_class(cls):
        return {
            "module": cls.__module__,
            "name": cls.__name__
        }

    def import_model(self, model_name):
        model = self.get_model(model_name)
        formatted_model = self.format_class(model)
        return formatted_model

    @staticmethod
    def get_app(cls):
        return cls._meta.app_label

    @staticmethod
    def get_model(model_name):
        models = django.apps.apps.get_models()
        for model in models:
            if model.__name__.lower() == model_name.lower():
                return model

    @staticmethod
    def get_model_fields(model):
        fields = [{"name": i.name, "cls": i} for i in model._meta.get_fields()]

        return fields



import os
import pathlib
import inflection
from jinja2 import Environment, FileSystemLoader


def format_with_jinja(data, template_src):
    base_folder = pathlib.Path(__file__).resolve().parent
    env = Environment(loader=FileSystemLoader(base_folder))
    env.filters['underscore'] = inflection.underscore
    env.filters['camelize'] = inflection.camelize
    env.filters['dasherize'] = inflection.dasherize
    env.filters['pluralize'] = inflection.pluralize
    env.filters['singularize'] = inflection.singularize
    env.filters['str'] = str

    with open(os.path.join(base_folder, 'templates', template_src), "r", encoding="utf-8") as f:
        template = env.from_string(f.read())

    result = template.render(**data)
    return result



import os


class CleanMigration:
    def __init__(self, app_name=None):
        self.app_name = app_name

    def run(self):
        target_dir = self.app_name if self.app_name else ""
        for root, dirs, files in os.walk("."):
            if "migrations" in root and "__pycache__" not in root and target_dir in root:
                for file in files:
                    if not file.startswith("__init__"):
                        os.remove(os.path.join(root, file))



import os


class ExtractModelsService:

    def run(self):
        files = self.get_models_and_serializer_files()
        result = []
        for file_path in files:
            with open(file_path, 'r', encoding='utf-8') as file:
                content = file.read().strip()
                if content:  # Ignore empty files
                    relative_path = os.path.relpath(file_path, os.getcwd())
                    result.append(f"# {relative_path}\n\n{content}\n")
        response = "\n".join(result)
        return response

    def get_models_and_serializer_files(self):
        project_root = os.getcwd()
        apps = self.get_apps(project_root)
        all_models_and_serializers = []

        for app in apps:
            models_path = os.path.join(project_root, app, 'models')
            serializers_path = os.path.join(project_root, app, 'serializers')

            if os.path.isdir(models_path):
                all_models_and_serializers.extend(self.get_python_files(models_path))
            elif os.path.exists(models_path + '.py'):
                all_models_and_serializers.append(models_path + '.py')

            if os.path.isdir(serializers_path):
                all_models_and_serializers.extend(self.get_python_files(serializers_path))
            elif os.path.exists(serializers_path + '.py'):
                all_models_and_serializers.append(serializers_path + '.py')

        return all_models_and_serializers

    @staticmethod
    def get_apps(project_root) -> list[str]:
        apps = []
        for root, dirs, files in os.walk(project_root):
            if 'apps.py' in files:
                app_path = os.path.relpath(root, project_root)
                apps.append(str(app_path))
        return apps

    @staticmethod
    def get_python_files(directory):
        python_files = []
        for root, _, files in os.walk(directory):
            for file in files:
                if file.endswith('.py'):
                    python_files.append(os.path.join(root, file))
        return python_files



from django.core.management.base import BaseCommand

from djx_cmds.services.scaffolding_service import ScaffoldingService


class Command(BaseCommand):
    help = ''

    def __init__(self):
        super(Command, self).__init__()
        self.available_methods = [
            {'code': 'c', 'action': 'Create'},
            {'code': 'r', 'action': ''},
            {'code': 'u', 'action': 'Update'},
            {'code': 'd', 'action': 'Delete'},
        ]

    def add_arguments(self, parser):
        parser.add_argument('--name', type=str, help='name')
        parser.add_argument('--action', type=str, default='r')

    def handle(self, *args, **options):
        name = options['name']
        allowed_actions = options['action']

        ScaffoldingService(name, allowed_actions).run()




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['djx_cmds',
 'djx_cmds.management',
 'djx_cmds.management.commands',
 'djx_cmds.migrations',
 'djx_cmds.services']

package_data = \
{'': ['*'], 'djx_cmds': ['templates/*']}

install_requires = \
['django>=4.1', 'inflection>=0.5.1,<0.6.0', 'jinja2>=3.1.2,<4.0.0']

setup_kwargs = {
    'name': 'djx-cmds',
    'version': '0.2.1',
    'description': '',
    'long_description': '',
    'author': 'nope',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import importlib
import inspect
import os
import pprint
from collections import defaultdict
from subprocess import check_output

import django
import inflection

from djx_cmds.format_with_jinja import format_with_jinja
from rest_framework import serializers


class ModelsToAngular:

    def __init__(self, file_path):
        self.file_path = file_path
        self.formly_type_map = {
            "TextField": "textarea",
            "BooleanField": "checkbox",
            "IntegerField": "input"
        }
        self.formly_props_type_map = {
            "DateField": "date",
            "IntegerField": "number",
            "FloatField": "number",
            "EmailField": "email",
        }

    def run(self):
        # self.check_client_folder()
        serializers_files = self.get_models_and_serializer_files()
        serializers_list = self.get_serializer_list(serializers_files)
        serializer_data = self.get_serializer_data(serializers_list)
        models_actions = self.get_models_and_actions(serializer_data)
        forms = self.generate_form_or_table(
            serializer_data, "FormlyFieldConfig", "formly.txt")
        tables = self.generate_form_or_table(serializer_data, "MtxGridColumn", "mtx_grid_table.txt")
        data = {**forms, **tables}
        self.configure_angular(models_actions)
        self.find_and_replace(self.file_path, data)

    def configure_angular(self, models_actions):
        all_command = [f'cd "{self.file_path}"']
        for element in models_actions:
            model_name = inflection.dasherize(models_actions[element]["model_name"]).lower()
            service_file_name = f"{model_name}.service.ts"
            if not self.check_if_file_exists(self.file_path, service_file_name):
                text = format_with_jinja(
                    models_actions[element],
                    "angular.txt"
                )
                all_command += text.split("\n")
        cmds = " && ".join(all_command)
        output = check_output(cmds, shell=True)
        print(output)

    @staticmethod
    def get_models_and_actions(serializer_data):
        action_data = defaultdict(list)
        for key in serializer_data:
            data = serializer_data[key]
            model_name = data['model_name']
            action = data["serializer_name"].replace(model_name, "").lower()
            action = action if action else "list"
            action_data[model_name].append(action)

        result = {serializer_data[key]["model_name"]: {
            "actions": action_data[serializer_data[key]["model_name"]],
            "model_name": serializer_data[key]["model_name"],
            "app_name": serializer_data[key]["app_name"]
        } for key in serializer_data}
        return result

    @staticmethod
    def check_if_file_exists(file_path, file_name):
        for root, dirs, files in os.walk(file_path):
            if file_name in files:
                return True
        return False

    @staticmethod
    def find_and_replace(file_path, data):
        for root, dirs, files in os.walk(file_path):
            for file_name in files:
                full_path = os.path.join(root, file_name)

                with open(full_path, 'r', encoding='utf-8') as file:
                    try:
                        file_content = file.read()
                    except UnicodeDecodeError:
                        continue

                modified_content = file_content
                for key, value in data.items():
                    modified_content = modified_content.replace(key, value)

                if modified_content != file_content:
                    with open(full_path, 'w', encoding='utf-8') as file:
                        file.write(modified_content)

    @staticmethod
    def generate_form_or_table(serializer_data, initial_tag, template_file):
        data = {}
        for key in serializer_data:
            text = format_with_jinja(
                serializer_data[key],
                template_file
            )
            tag = f"// TAG: {key}{initial_tag}"
            data[tag] = text.strip()
            print(text)
        return data

    @staticmethod
    def get_serializer_list(files):
        serializers_list = []
        for file_path in files["serializers"]:
            module_name = os.path.splitext(os.path.basename(file_path))[0]
            spec = importlib.util.spec_from_file_location(module_name, file_path)
            module = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(module)
            for name, obj in inspect.getmembers(module):
                if inspect.isclass(obj) and (
                        issubclass(obj, serializers.Serializer) or issubclass(obj, serializers.ModelSerializer)):
                    serializers_list.append(obj)
        return serializers_list

    def get_serializer_data(self, serializer_list):
        serializer_fields = {}
        for serializer in serializer_list:
            serializer_instance = serializer()
            model = getattr(serializer.Meta, 'model', None) if hasattr(serializer, 'Meta') and hasattr(serializer.Meta,
                                                                                                       'model') else None
            model_name = model.__name__ if model else None
            app_name = model._meta.app_label if model else None
            serializer_name = f"{serializer.__name__}".replace("Serializer", "")

            if hasattr(serializer_instance, 'fields'):
                fields = serializer_instance.fields
                field_objects = [
                    {
                        'name': field_name,
                        'type': type(field).__name__,
                        'required': field.required,
                        'write_only': field.write_only,
                        'read_only': field.read_only,
                        'nullable': field.allow_null,
                        'max_length': getattr(field, 'max_length', None),
                        'size': getattr(field, 'size', None),
                        'help_text': getattr(field, 'help_text', None),
                        'is_primary_key': field.source in ['id', 'pk'],
                    }
                    for field_name, field in fields.items()
                ]
                serializer_fields[serializer_name] = {
                    "fields": field_objects,
                    "model_name": model_name,
                    "serializer_name": serializer_name,
                    "app_name": app_name,
                    "formly_type_map": self.formly_type_map,
                    "formly_props_type_map": self.formly_props_type_map,
                }
        return serializer_fields

    def get_models_and_serializer_files(self):
        project_root = os.getcwd()
        apps = self.get_apps(project_root)
        models = []
        serializers = []
        for app in apps:
            models_path = os.path.join(project_root, app, 'models')
            serializers_path = os.path.join(project_root, app, 'serializers')

            if os.path.isdir(models_path):
                models.extend(self.get_python_files(models_path))
            elif os.path.exists(models_path + '.py'):
                models.append(models_path + '.py')

            if os.path.isdir(serializers_path):
                serializers.extend(self.get_python_files(serializers_path))
            elif os.path.exists(serializers_path + '.py'):
                serializers.append(serializers_path + '.py')

        return {
            "models": models,
            "serializers": serializers
        }

    @staticmethod
    def get_apps(project_root) -> list[str]:
        apps = []
        for root, dirs, files in os.walk(project_root):
            if 'apps.py' in files:
                app_path = os.path.relpath(root, project_root)
                apps.append(str(app_path))
        return apps

    @staticmethod
    def get_python_files(directory):
        python_files = []
        for root, _, files in os.walk(directory):
            for file in files:
                if file.endswith('.py'):
                    python_files.append(os.path.join(root, file))
        return python_files

    def check_client_folder(self):
        if not os.path.exists(self.file_path) or not os.path.isdir(self.file_path):
            raise FileNotFoundError(f"The folder '{self.file_path}' does not exist.")
        if os.path.abspath(self.file_path) == os.path.abspath(os.sep) or not os.listdir(self.file_path):
            raise ValueError(f"The folder '{self.file_path}' is either root or empty.")
